@@ -1,8 +1,9 @@
 //===-- CodeGen/MachineFrameInfo.h - Abstract Stack Frame Rep. --*- C++ -*-===//
 //
-// Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
-// See https://llvm.org/LICENSE.txt for license information.
-// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
+//                     The LLVM Compiler Infrastructure
+//
+// This file is distributed under the University of Illinois Open Source
+// License. See LICENSE.TXT for details.
 //
 //===----------------------------------------------------------------------===//
 //
@@ -14,64 +15,37 @@
 #define LLVM_CODEGEN_MACHINEFRAMEINFO_H
 
 #include "llvm/ADT/SmallVector.h"
-#include "llvm/Support/Alignment.h"
 #include "llvm/Support/DataTypes.h"
 #include <cassert>
 #include <vector>
 
 namespace llvm {
 class raw_ostream;
+class DataLayout;
+class TargetRegisterClass;
+class Type;
 class MachineFunction;
 class MachineBasicBlock;
+class TargetFrameLowering;
+class TargetMachine;
 class BitVector;
+class Value;
 class AllocaInst;
 
 /// The CalleeSavedInfo class tracks the information need to locate where a
 /// callee saved register is in the current frame.
-/// Callee saved reg can also be saved to a different register rather than
-/// on the stack by setting DstReg instead of FrameIdx.
 class CalleeSavedInfo {
   unsigned Reg;
-  union {
-    int FrameIdx;
-    unsigned DstReg;
-  };
-  /// Flag indicating whether the register is actually restored in the epilog.
-  /// In most cases, if a register is saved, it is also restored. There are
-  /// some situations, though, when this is not the case. For example, the
-  /// LR register on ARM is usually saved, but on exit from the function its
-  /// saved value may be loaded directly into PC. Since liveness tracking of
-  /// physical registers treats callee-saved registers are live outside of
-  /// the function, LR would be treated as live-on-exit, even though in these
-  /// scenarios it is not. This flag is added to indicate that the saved
-  /// register described by this object is not restored in the epilog.
-  /// The long-term solution is to model the liveness of callee-saved registers
-  /// by implicit uses on the return instructions, however, the required
-  /// changes in the ARM backend would be quite extensive.
-  bool Restored;
-  /// Flag indicating whether the register is spilled to stack or another
-  /// register.
-  bool SpilledToReg;
+  int FrameIdx;
 
 public:
   explicit CalleeSavedInfo(unsigned R, int FI = 0)
-  : Reg(R), FrameIdx(FI), Restored(true), SpilledToReg(false) {}
+  : Reg(R), FrameIdx(FI) {}
 
   // Accessors.
   unsigned getReg()                        const { return Reg; }
   int getFrameIdx()                        const { return FrameIdx; }
-  unsigned getDstReg()                     const { return DstReg; }
-  void setFrameIdx(int FI) {
-    FrameIdx = FI;
-    SpilledToReg = false;
-  }
-  void setDstReg(unsigned SpillReg) {
-    DstReg = SpillReg;
-    SpilledToReg = true;
-  }
-  bool isRestored()                        const { return Restored; }
-  void setRestored(bool R)                       { Restored = R; }
-  bool isSpilledToReg()                    const { return SpilledToReg; }
+  void setFrameIdx(int FI)                       { FrameIdx = FI; }
 };
 
 /// The MachineFrameInfo class represents an abstract stack frame until
@@ -102,23 +76,9 @@ public:
 /// stack offsets of the object, eliminating all MO_FrameIndex operands from
 /// the program.
 ///
-/// Abstract Stack Frame Information
+/// @brief Abstract Stack Frame Information
 class MachineFrameInfo {
-public:
-  /// Stack Smashing Protection (SSP) rules require that vulnerable stack
-  /// allocations are located close the stack protector.
-  enum SSPLayoutKind {
-    SSPLK_None,       ///< Did not trigger a stack protector.  No effect on data
-                      ///< layout.
-    SSPLK_LargeArray, ///< Array or nested array >= SSP-buffer-size.  Closest
-                      ///< to the stack protector.
-    SSPLK_SmallArray, ///< Array or nested array < SSP-buffer-size. 2nd closest
-                      ///< to the stack protector.
-    SSPLK_AddrOf      ///< The address of this allocation is exposed and
-                      ///< triggered protection.  3rd closest to the protector.
-  };
 
-private:
   // Represent a single object allocated on the stack.
   struct StackObject {
     // The offset of this object from the stack pointer on entry to
@@ -130,7 +90,7 @@ private:
     uint64_t Size;
 
     // The required alignment of this stack slot.
-    Align Alignment;
+    unsigned Alignment;
 
     // If true, the value of the stack object is set before
     // entering the function and is not modified inside the function. By
@@ -141,31 +101,13 @@ private:
     // cannot alias any other memory objects.
     bool isSpillSlot;
 
-    /// If true, this stack slot is used to spill a value (could be deopt
-    /// and/or GC related) over a statepoint. We know that the address of the
-    /// slot can't alias any LLVM IR value.  This is very similar to a Spill
-    /// Slot, but is created by statepoint lowering is SelectionDAG, not the
-    /// register allocator.
-    bool isStatepointSpillSlot = false;
-
-    /// Identifier for stack memory type analagous to address space. If this is
-    /// non-0, the meaning is target defined. Offsets cannot be directly
-    /// compared between objects with different stack IDs. The object may not
-    /// necessarily reside in the same contiguous memory block as other stack
-    /// objects. Objects with differing stack IDs should not be merged or
-    /// replaced substituted for each other.
-    //
-    /// It is assumed a target uses consecutive, increasing stack IDs starting
-    /// from 1.
-    uint8_t StackID;
-
     /// If this stack object is originated from an Alloca instruction
     /// this value saves the original IR allocation. Can be NULL.
     const AllocaInst *Alloca;
 
     // If true, the object was mapped into the local frame
     // block and doesn't need additional handling for allocation beyond that.
-    bool PreAllocated = false;
+    bool PreAllocated;
 
     // If true, an LLVM IR value might point to this object.
     // Normally, spill slots and fixed-offset objects don't alias IR-accessible
@@ -173,74 +115,51 @@ private:
     // arguments have ABI-prescribed offsets).
     bool isAliased;
 
-    /// If true, the object has been zero-extended.
-    bool isZExt = false;
-
-    /// If true, the object has been zero-extended.
-    bool isSExt = false;
-
-    uint8_t SSPLayout;
-
-    StackObject(uint64_t Size, Align Alignment, int64_t SPOffset,
-                bool IsImmutable, bool IsSpillSlot, const AllocaInst *Alloca,
-                bool IsAliased, uint8_t StackID = 0)
-        : SPOffset(SPOffset), Size(Size), Alignment(Alignment),
-          isImmutable(IsImmutable), isSpillSlot(IsSpillSlot), StackID(StackID),
-          Alloca(Alloca), isAliased(IsAliased), SSPLayout(SSPLK_None) {}
+    StackObject(uint64_t Sz, unsigned Al, int64_t SP, bool IM,
+                bool isSS, const AllocaInst *Val, bool A)
+      : SPOffset(SP), Size(Sz), Alignment(Al), isImmutable(IM),
+        isSpillSlot(isSS), Alloca(Val), PreAllocated(false), isAliased(A) {}
   };
 
   /// The alignment of the stack.
-  Align StackAlignment;
+  unsigned StackAlignment;
 
-  /// Can the stack be realigned. This can be false if the target does not
-  /// support stack realignment, or if the user asks us not to realign the
-  /// stack. In this situation, overaligned allocas are all treated as dynamic
-  /// allocations and the target must handle them as part of DYNAMIC_STACKALLOC
-  /// lowering. All non-alloca stack objects have their alignment clamped to the
-  /// base ABI stack alignment.
-  /// FIXME: There is room for improvement in this case, in terms of
-  /// grouping overaligned allocas into a "secondary stack frame" and
-  /// then only use a single alloca to allocate this frame and only a
-  /// single virtual register to access it. Currently, without such an
-  /// optimization, each such alloca gets its own dynamic realignment.
+  /// Can the stack be realigned.
   bool StackRealignable;
 
-  /// Whether the function has the \c alignstack attribute.
-  bool ForcedRealign;
-
   /// The list of stack objects allocated.
   std::vector<StackObject> Objects;
 
   /// This contains the number of fixed objects contained on
   /// the stack.  Because fixed objects are stored at a negative index in the
   /// Objects list, this is also the index to the 0th object in the list.
-  unsigned NumFixedObjects = 0;
+  unsigned NumFixedObjects;
 
   /// This boolean keeps track of whether any variable
   /// sized objects have been allocated yet.
-  bool HasVarSizedObjects = false;
+  bool HasVarSizedObjects;
 
   /// This boolean keeps track of whether there is a call
   /// to builtin \@llvm.frameaddress.
-  bool FrameAddressTaken = false;
+  bool FrameAddressTaken;
 
   /// This boolean keeps track of whether there is a call
   /// to builtin \@llvm.returnaddress.
-  bool ReturnAddressTaken = false;
+  bool ReturnAddressTaken;
 
   /// This boolean keeps track of whether there is a call
   /// to builtin \@llvm.experimental.stackmap.
-  bool HasStackMap = false;
+  bool HasStackMap;
 
   /// This boolean keeps track of whether there is a call
   /// to builtin \@llvm.experimental.patchpoint.
-  bool HasPatchPoint = false;
+  bool HasPatchPoint;
 
   /// The prolog/epilog code inserter calculates the final stack
   /// offsets for all of the fixed size objects, updating the Objects list
   /// above.  It then updates StackSize to contain the number of bytes that need
   /// to be allocated on entry to the function.
-  uint64_t StackSize = 0;
+  uint64_t StackSize;
 
   /// The amount that a frame offset needs to be adjusted to
   /// have the actual offset from the stack/frame pointer.  The exact usage of
@@ -249,9 +168,9 @@ private:
   /// SP then OffsetAdjustment is zero; if FP is used, OffsetAdjustment is set
   /// to the distance between the initial SP and the value in FP.  For many
   /// targets, this value is only used when generating debug info (via
-  /// TargetRegisterInfo::getFrameIndexReference); when generating code, the
+  /// TargetRegisterInfo::getFrameIndexOffset); when generating code, the
   /// corresponding adjustments are performed directly.
-  int OffsetAdjustment = 0;
+  int OffsetAdjustment;
 
   /// The prolog/epilog code inserter may process objects that require greater
   /// alignment than the default alignment the target provides.
@@ -260,86 +179,103 @@ private:
   /// native alignment maintained by the compiler, dynamic alignment code will
   /// be needed.
   ///
-  Align MaxAlignment;
+  unsigned MaxAlignment;
 
   /// Set to true if this function adjusts the stack -- e.g.,
   /// when calling another function. This is only valid during and after
   /// prolog/epilog code insertion.
-  bool AdjustsStack = false;
+  bool AdjustsStack;
 
   /// Set to true if this function has any function calls.
-  bool HasCalls = false;
+  bool HasCalls;
 
   /// The frame index for the stack protector.
-  int StackProtectorIdx = -1;
+  int StackProtectorIdx;
 
   /// The frame index for the function context. Used for SjLj exceptions.
-  int FunctionContextIdx = -1;
+  int FunctionContextIdx;
 
   /// This contains the size of the largest call frame if the target uses frame
   /// setup/destroy pseudo instructions (as defined in the TargetFrameInfo
   /// class).  This information is important for frame pointer elimination.
-  /// It is only valid during and after prolog/epilog code insertion.
-  unsigned MaxCallFrameSize = ~0u;
-
-  /// The number of bytes of callee saved registers that the target wants to
-  /// report for the current function in the CodeView S_FRAMEPROC record.
-  unsigned CVBytesOfCalleeSavedRegisters = 0;
+  /// If is only valid during and after prolog/epilog code insertion.
+  unsigned MaxCallFrameSize;
 
   /// The prolog/epilog code inserter fills in this vector with each
-  /// callee saved register saved in either the frame or a different
-  /// register.  Beyond its use by the prolog/ epilog code inserter,
-  /// this data is used for debug info and exception handling.
+  /// callee saved register saved in the frame.  Beyond its use by the prolog/
+  /// epilog code inserter, this data used for debug info and exception
+  /// handling.
   std::vector<CalleeSavedInfo> CSInfo;
 
   /// Has CSInfo been set yet?
-  bool CSIValid = false;
+  bool CSIValid;
 
   /// References to frame indices which are mapped
   /// into the local frame allocation block. <FrameIdx, LocalOffset>
   SmallVector<std::pair<int, int64_t>, 32> LocalFrameObjects;
 
   /// Size of the pre-allocated local frame block.
-  int64_t LocalFrameSize = 0;
+  int64_t LocalFrameSize;
 
   /// Required alignment of the local object blob, which is the strictest
   /// alignment of any object in it.
-  Align LocalFrameMaxAlign;
+  unsigned LocalFrameMaxAlign;
 
   /// Whether the local object blob needs to be allocated together. If not,
   /// PEI should ignore the isPreAllocated flags on the stack objects and
   /// just allocate them normally.
-  bool UseLocalStackAllocationBlock = false;
+  bool UseLocalStackAllocationBlock;
+
+  /// Whether the "realign-stack" option is on.
+  bool RealignOption;
 
   /// True if the function dynamically adjusts the stack pointer through some
   /// opaque mechanism like inline assembly or Win32 EH.
-  bool HasOpaqueSPAdjustment = false;
-
-  /// True if the function contains operations which will lower down to
-  /// instructions which manipulate the stack pointer.
-  bool HasCopyImplyingStackAdjustment = false;
+  bool HasOpaqueSPAdjustment;
 
   /// True if the function contains a call to the llvm.vastart intrinsic.
-  bool HasVAStart = false;
+  bool HasVAStart;
 
   /// True if this is a varargs function that contains a musttail call.
-  bool HasMustTailInVarArgFunc = false;
+  bool HasMustTailInVarArgFunc;
 
   /// True if this function contains a tail call. If so immutable objects like
   /// function arguments are no longer so. A tail call *can* override fixed
   /// stack objects like arguments so we can't treat them as immutable.
-  bool HasTailCall = false;
+  bool HasTailCall;
 
   /// Not null, if shrink-wrapping found a better place for the prologue.
-  MachineBasicBlock *Save = nullptr;
+  MachineBasicBlock *Save;
   /// Not null, if shrink-wrapping found a better place for the epilogue.
-  MachineBasicBlock *Restore = nullptr;
+  MachineBasicBlock *Restore;
 
 public:
-  explicit MachineFrameInfo(unsigned StackAlignment, bool StackRealignable,
-                            bool ForcedRealign)
-      : StackAlignment(assumeAligned(StackAlignment)),
-        StackRealignable(StackRealignable), ForcedRealign(ForcedRealign) {}
+  explicit MachineFrameInfo(unsigned StackAlign, bool isStackRealign,
+                            bool RealignOpt)
+      : StackAlignment(StackAlign), StackRealignable(isStackRealign),
+        RealignOption(RealignOpt) {
+    StackSize = NumFixedObjects = OffsetAdjustment = MaxAlignment = 0;
+    HasVarSizedObjects = false;
+    FrameAddressTaken = false;
+    ReturnAddressTaken = false;
+    HasStackMap = false;
+    HasPatchPoint = false;
+    AdjustsStack = false;
+    HasCalls = false;
+    StackProtectorIdx = -1;
+    FunctionContextIdx = -1;
+    MaxCallFrameSize = 0;
+    CSIValid = false;
+    LocalFrameSize = 0;
+    LocalFrameMaxAlign = 0;
+    UseLocalStackAllocationBlock = false;
+    HasOpaqueSPAdjustment = false;
+    HasVAStart = false;
+    HasMustTailInVarArgFunc = false;
+    Save = nullptr;
+    Restore = nullptr;
+    HasTailCall = false;
+  }
 
   /// Return true if there are any stack objects in this function.
   bool hasStackObjects() const { return !Objects.empty(); }
@@ -352,7 +288,6 @@ public:
   /// Return the index for the stack protector object.
   int getStackProtectorIndex() const { return StackProtectorIdx; }
   void setStackProtectorIndex(int I) { StackProtectorIdx = I; }
-  bool hasStackProtectorIndex() const { return StackProtectorIdx != -1; }
 
   /// Return the index for the function context object.
   /// This object is used for SjLj exceptions.
@@ -402,14 +337,14 @@ public:
   }
 
   /// Get the local offset mapping for a for an object.
-  std::pair<int, int64_t> getLocalFrameObjectMap(int i) const {
+  std::pair<int, int64_t> getLocalFrameObjectMap(int i) {
     assert (i >= 0 && (unsigned)i < LocalFrameObjects.size() &&
             "Invalid local object reference!");
     return LocalFrameObjects[i];
   }
 
   /// Return the number of objects allocated into the local object block.
-  int64_t getLocalFrameObjectCount() const { return LocalFrameObjects.size(); }
+  int64_t getLocalFrameObjectCount() { return LocalFrameObjects.size(); }
 
   /// Set the size of the local object blob.
   void setLocalFrameSize(int64_t sz) { LocalFrameSize = sz; }
@@ -419,18 +354,14 @@ public:
 
   /// Required alignment of the local object blob,
   /// which is the strictest alignment of any object in it.
-  void setLocalFrameMaxAlign(Align Alignment) {
-    LocalFrameMaxAlign = Alignment;
-  }
+  void setLocalFrameMaxAlign(unsigned Align) { LocalFrameMaxAlign = Align; }
 
   /// Return the required alignment of the local object blob.
-  Align getLocalFrameMaxAlign() const { return LocalFrameMaxAlign; }
+  unsigned getLocalFrameMaxAlign() const { return LocalFrameMaxAlign; }
 
   /// Get whether the local allocation blob should be allocated together or
   /// let PEI allocate the locals in it directly.
-  bool getUseLocalStackAllocationBlock() const {
-    return UseLocalStackAllocationBlock;
-  }
+  bool getUseLocalStackAllocationBlock() {return UseLocalStackAllocationBlock;}
 
   /// setUseLocalStackAllocationBlock - Set whether the local allocation blob
   /// should be allocated together or let PEI allocate the locals in it
@@ -464,18 +395,15 @@ public:
   unsigned getObjectAlignment(int ObjectIdx) const {
     assert(unsigned(ObjectIdx+NumFixedObjects) < Objects.size() &&
            "Invalid Object Idx!");
-    return Objects[ObjectIdx + NumFixedObjects].Alignment.value();
+    return Objects[ObjectIdx+NumFixedObjects].Alignment;
   }
 
   /// setObjectAlignment - Change the alignment of the specified stack object.
   void setObjectAlignment(int ObjectIdx, unsigned Align) {
     assert(unsigned(ObjectIdx+NumFixedObjects) < Objects.size() &&
            "Invalid Object Idx!");
-    Objects[ObjectIdx + NumFixedObjects].Alignment = assumeAligned(Align);
-
-    // Only ensure max alignment for the default stack.
-    if (getStackID(ObjectIdx) == 0)
-      ensureMaxAlignment(Align);
+    Objects[ObjectIdx+NumFixedObjects].Alignment = Align;
+    ensureMaxAlignment(Align);
   }
 
   /// Return the underlying Alloca of the specified
@@ -496,30 +424,6 @@ public:
     return Objects[ObjectIdx+NumFixedObjects].SPOffset;
   }
 
-  bool isObjectZExt(int ObjectIdx) const {
-    assert(unsigned(ObjectIdx+NumFixedObjects) < Objects.size() &&
-           "Invalid Object Idx!");
-    return Objects[ObjectIdx+NumFixedObjects].isZExt;
-  }
-
-  void setObjectZExt(int ObjectIdx, bool IsZExt) {
-    assert(unsigned(ObjectIdx+NumFixedObjects) < Objects.size() &&
-           "Invalid Object Idx!");
-    Objects[ObjectIdx+NumFixedObjects].isZExt = IsZExt;
-  }
-
-  bool isObjectSExt(int ObjectIdx) const {
-    assert(unsigned(ObjectIdx+NumFixedObjects) < Objects.size() &&
-           "Invalid Object Idx!");
-    return Objects[ObjectIdx+NumFixedObjects].isSExt;
-  }
-
-  void setObjectSExt(int ObjectIdx, bool IsSExt) {
-    assert(unsigned(ObjectIdx+NumFixedObjects) < Objects.size() &&
-           "Invalid Object Idx!");
-    Objects[ObjectIdx+NumFixedObjects].isSExt = IsSExt;
-  }
-
   /// Set the stack frame offset of the specified object. The
   /// offset is relative to the stack pointer on entry to the function.
   void setObjectOffset(int ObjectIdx, int64_t SPOffset) {
@@ -530,20 +434,6 @@ public:
     Objects[ObjectIdx+NumFixedObjects].SPOffset = SPOffset;
   }
 
-  SSPLayoutKind getObjectSSPLayout(int ObjectIdx) const {
-    assert(unsigned(ObjectIdx+NumFixedObjects) < Objects.size() &&
-           "Invalid Object Idx!");
-    return (SSPLayoutKind)Objects[ObjectIdx+NumFixedObjects].SSPLayout;
-  }
-
-  void setObjectSSPLayout(int ObjectIdx, SSPLayoutKind Kind) {
-    assert(unsigned(ObjectIdx+NumFixedObjects) < Objects.size() &&
-           "Invalid Object Idx!");
-    assert(!isDeadObjectIndex(ObjectIdx) &&
-           "Setting SSP layout for a dead object?");
-    Objects[ObjectIdx+NumFixedObjects].SSPLayout = Kind;
-  }
-
   /// Return the number of bytes that must be allocated to hold
   /// all of the fixed size frame objects.  This is only valid after
   /// Prolog/Epilog code insertion has finalized the stack frame layout.
@@ -553,7 +443,7 @@ public:
   void setStackSize(uint64_t Size) { StackSize = Size; }
 
   /// Estimate and return the size of the stack frame.
-  uint64_t estimateStackSize(const MachineFunction &MF) const;
+  unsigned estimateStackSize(const MachineFunction &MF) const;
 
   /// Return the correction for frame offsets.
   int getOffsetAdjustment() const { return OffsetAdjustment; }
@@ -563,14 +453,10 @@ public:
 
   /// Return the alignment in bytes that this function must be aligned to,
   /// which is greater than the default stack alignment provided by the target.
-  unsigned getMaxAlignment() const { return MaxAlignment.value(); }
+  unsigned getMaxAlignment() const { return MaxAlignment; }
 
   /// Make sure the function is at least Align bytes aligned.
-  void ensureMaxAlignment(Align Alignment);
-  /// FIXME: Remove this once transition to Align is over.
-  inline void ensureMaxAlignment(unsigned Align) {
-    ensureMaxAlignment(assumeAligned(Align));
-  }
+  void ensureMaxAlignment(unsigned Align);
 
   /// Return true if this function adjusts the stack -- e.g.,
   /// when calling another function. This is only valid during and after
@@ -586,15 +472,6 @@ public:
   bool hasOpaqueSPAdjustment() const { return HasOpaqueSPAdjustment; }
   void setHasOpaqueSPAdjustment(bool B) { HasOpaqueSPAdjustment = B; }
 
-  /// Returns true if the function contains operations which will lower down to
-  /// instructions which manipulate the stack pointer.
-  bool hasCopyImplyingStackAdjustment() const {
-    return HasCopyImplyingStackAdjustment;
-  }
-  void setHasCopyImplyingStackAdjustment(bool B) {
-    HasCopyImplyingStackAdjustment = B;
-  }
-
   /// Returns true if the function calls the llvm.va_start intrinsic.
   bool hasVAStart() const { return HasVAStart; }
   void setHasVAStart(bool B) { HasVAStart = B; }
@@ -607,51 +484,24 @@ public:
   bool hasTailCall() const { return HasTailCall; }
   void setHasTailCall() { HasTailCall = true; }
 
-  /// Computes the maximum size of a callframe and the AdjustsStack property.
-  /// This only works for targets defining
-  /// TargetInstrInfo::getCallFrameSetupOpcode(), getCallFrameDestroyOpcode(),
-  /// and getFrameSize().
-  /// This is usually computed by the prologue epilogue inserter but some
-  /// targets may call this to compute it earlier.
-  void computeMaxCallFrameSize(const MachineFunction &MF);
-
   /// Return the maximum size of a call frame that must be
   /// allocated for an outgoing function call.  This is only available if
   /// CallFrameSetup/Destroy pseudo instructions are used by the target, and
   /// then only during or after prolog/epilog code insertion.
   ///
-  unsigned getMaxCallFrameSize() const {
-    // TODO: Enable this assert when targets are fixed.
-    //assert(isMaxCallFrameSizeComputed() && "MaxCallFrameSize not computed yet");
-    if (!isMaxCallFrameSizeComputed())
-      return 0;
-    return MaxCallFrameSize;
-  }
-  bool isMaxCallFrameSizeComputed() const {
-    return MaxCallFrameSize != ~0u;
-  }
+  unsigned getMaxCallFrameSize() const { return MaxCallFrameSize; }
   void setMaxCallFrameSize(unsigned S) { MaxCallFrameSize = S; }
 
-  /// Returns how many bytes of callee-saved registers the target pushed in the
-  /// prologue. Only used for debug info.
-  unsigned getCVBytesOfCalleeSavedRegisters() const {
-    return CVBytesOfCalleeSavedRegisters;
-  }
-  void setCVBytesOfCalleeSavedRegisters(unsigned S) {
-    CVBytesOfCalleeSavedRegisters = S;
-  }
-
   /// Create a new object at a fixed location on the stack.
   /// All fixed objects should be created before other objects are created for
   /// efficiency. By default, fixed objects are not pointed to by LLVM IR
   /// values. This returns an index with a negative value.
-  int CreateFixedObject(uint64_t Size, int64_t SPOffset, bool IsImmutable,
+  int CreateFixedObject(uint64_t Size, int64_t SPOffset, bool Immutable,
                         bool isAliased = false);
 
   /// Create a spill slot at a fixed location on the stack.
   /// Returns an index with a negative value.
-  int CreateFixedSpillStackObject(uint64_t Size, int64_t SPOffset,
-                                  bool IsImmutable = false);
+  int CreateFixedSpillStackObject(uint64_t Size, int64_t SPOffset);
 
   /// Returns true if the specified index corresponds to a fixed stack object.
   bool isFixedObjectIndex(int ObjectIdx) const {
@@ -666,7 +516,8 @@ public:
     return Objects[ObjectIdx+NumFixedObjects].isAliased;
   }
 
-  /// Returns true if the specified index corresponds to an immutable object.
+  /// isImmutableObjectIndex - Returns true if the specified index corresponds
+  /// to an immutable object.
   bool isImmutableObjectIndex(int ObjectIdx) const {
     // Tail calling functions can clobber their function arguments.
     if (HasTailCall)
@@ -676,13 +527,6 @@ public:
     return Objects[ObjectIdx+NumFixedObjects].isImmutable;
   }
 
-  /// Marks the immutability of an object.
-  void setIsImmutableObjectIndex(int ObjectIdx, bool IsImmutable) {
-    assert(unsigned(ObjectIdx+NumFixedObjects) < Objects.size() &&
-           "Invalid Object Idx!");
-    Objects[ObjectIdx+NumFixedObjects].isImmutable = IsImmutable;
-  }
-
   /// Returns true if the specified index corresponds to a spill slot.
   bool isSpillSlotObjectIndex(int ObjectIdx) const {
     assert(unsigned(ObjectIdx+NumFixedObjects) < Objects.size() &&
@@ -690,26 +534,6 @@ public:
     return Objects[ObjectIdx+NumFixedObjects].isSpillSlot;
   }
 
-  bool isStatepointSpillSlotObjectIndex(int ObjectIdx) const {
-    assert(unsigned(ObjectIdx+NumFixedObjects) < Objects.size() &&
-           "Invalid Object Idx!");
-    return Objects[ObjectIdx+NumFixedObjects].isStatepointSpillSlot;
-  }
-
-  /// \see StackID
-  uint8_t getStackID(int ObjectIdx) const {
-    return Objects[ObjectIdx+NumFixedObjects].StackID;
-  }
-
-  /// \see StackID
-  void setStackID(int ObjectIdx, uint8_t ID) {
-    assert(unsigned(ObjectIdx+NumFixedObjects) < Objects.size() &&
-           "Invalid Object Idx!");
-    Objects[ObjectIdx+NumFixedObjects].StackID = ID;
-    // If ID > 0, MaxAlignment may now be overly conservative.
-    // If ID == 0, MaxAlignment will need to be updated separately.
-  }
-
   /// Returns true if the specified index corresponds to a dead object.
   bool isDeadObjectIndex(int ObjectIdx) const {
     assert(unsigned(ObjectIdx+NumFixedObjects) < Objects.size() &&
@@ -725,33 +549,14 @@ public:
     return Objects[ObjectIdx + NumFixedObjects].Size == 0;
   }
 
-  void markAsStatepointSpillSlotObjectIndex(int ObjectIdx) {
-    assert(unsigned(ObjectIdx+NumFixedObjects) < Objects.size() &&
-           "Invalid Object Idx!");
-    Objects[ObjectIdx+NumFixedObjects].isStatepointSpillSlot = true;
-    assert(isStatepointSpillSlotObjectIndex(ObjectIdx) && "inconsistent");
-  }
-
   /// Create a new statically sized stack object, returning
   /// a nonnegative identifier to represent it.
-  int CreateStackObject(uint64_t Size, Align Alignment, bool isSpillSlot,
-                        const AllocaInst *Alloca = nullptr, uint8_t ID = 0);
-  /// FIXME: Remove this function when transition to Align is over.
-  inline int CreateStackObject(uint64_t Size, unsigned Alignment,
-                               bool isSpillSlot,
-                               const AllocaInst *Alloca = nullptr,
-                               uint8_t ID = 0) {
-    return CreateStackObject(Size, assumeAligned(Alignment), isSpillSlot,
-                             Alloca, ID);
-  }
+  int CreateStackObject(uint64_t Size, unsigned Alignment, bool isSS,
+                        const AllocaInst *Alloca = nullptr);
 
   /// Create a new statically sized stack object that represents a spill slot,
   /// returning a nonnegative identifier to represent it.
-  int CreateSpillStackObject(uint64_t Size, Align Alignment);
-  /// FIXME: Remove this function when transition to Align is over.
-  inline int CreateSpillStackObject(uint64_t Size, unsigned Alignment) {
-    return CreateSpillStackObject(Size, assumeAligned(Alignment));
-  }
+  int CreateSpillStackObject(uint64_t Size, unsigned Alignment);
 
   /// Remove or mark dead a statically sized stack object.
   void RemoveStackObject(int ObjectIdx) {
@@ -762,18 +567,12 @@ public:
   /// Notify the MachineFrameInfo object that a variable sized object has been
   /// created.  This must be created whenever a variable sized object is
   /// created, whether or not the index returned is actually used.
-  int CreateVariableSizedObject(Align Alignment, const AllocaInst *Alloca);
-  /// FIXME: Remove this function when transition to Align is over.
-  int CreateVariableSizedObject(unsigned Alignment, const AllocaInst *Alloca) {
-    return CreateVariableSizedObject(assumeAligned(Alignment), Alloca);
-  }
+  int CreateVariableSizedObject(unsigned Alignment, const AllocaInst *Alloca);
 
   /// Returns a reference to call saved info vector for the current function.
   const std::vector<CalleeSavedInfo> &getCalleeSavedInfo() const {
     return CSInfo;
   }
-  /// \copydoc getCalleeSavedInfo()
-  std::vector<CalleeSavedInfo> &getCalleeSavedInfo() { return CSInfo; }
 
   /// Used by prolog/epilog inserter to set the function's callee saved
   /// information.
