@@ -1,8 +1,9 @@
 //===- llvm/CodeGen/SlotIndexes.h - Slot indexes representation -*- C++ -*-===//
 //
-// Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
-// See https://llvm.org/LICENSE.txt for license information.
-// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
+//                     The LLVM Compiler Infrastructure
+//
+// This file is distributed under the University of Illinois Open Source
+// License. See LICENSE.TXT for details.
 //
 //===----------------------------------------------------------------------===//
 //
@@ -23,21 +24,13 @@
 #include "llvm/ADT/PointerIntPair.h"
 #include "llvm/ADT/SmallVector.h"
 #include "llvm/ADT/ilist.h"
-#include "llvm/CodeGen/MachineBasicBlock.h"
 #include "llvm/CodeGen/MachineFunction.h"
 #include "llvm/CodeGen/MachineFunctionPass.h"
-#include "llvm/CodeGen/MachineInstr.h"
 #include "llvm/CodeGen/MachineInstrBundle.h"
 #include "llvm/Support/Allocator.h"
-#include <algorithm>
-#include <cassert>
-#include <iterator>
-#include <utility>
 
 namespace llvm {
 
-class raw_ostream;
-
   /// This class represents an entry in the slot index list held in the
   /// SlotIndexes pass. It should not be used directly. See the
   /// SlotIndex & SlotIndexes classes for the public interface to this
@@ -47,6 +40,7 @@ class raw_ostream;
     unsigned index;
 
   public:
+
     IndexListEntry(MachineInstr *mi, unsigned index) : mi(mi), index(index) {}
 
     MachineInstr* getInstr() const { return mi; }
@@ -72,11 +66,27 @@ class raw_ostream;
 
     bool isPoisoned() const { return (reinterpret_cast<intptr_t>(mi) & 0x1) == 0x1; }
 #endif // EXPENSIVE_CHECKS
+
   };
 
   template <>
-  struct ilist_alloc_traits<IndexListEntry>
-      : public ilist_noalloc_traits<IndexListEntry> {};
+  struct ilist_traits<IndexListEntry> : public ilist_default_traits<IndexListEntry> {
+  private:
+    mutable ilist_half_node<IndexListEntry> Sentinel;
+  public:
+    IndexListEntry *createSentinel() const {
+      return static_cast<IndexListEntry*>(&Sentinel);
+    }
+    void destroySentinel(IndexListEntry *) const {}
+
+    IndexListEntry *provideInitialHead() const { return createSentinel(); }
+    IndexListEntry *ensureHead(IndexListEntry*) const { return createSentinel(); }
+    static void noteHead(IndexListEntry*, IndexListEntry*) {}
+    void deleteNode(IndexListEntry *N) {}
+
+  private:
+    void createNode(const IndexListEntry &);
+  };
 
   /// SlotIndex - An opaque wrapper around machine indexes.
   class SlotIndex {
@@ -89,7 +99,7 @@ class raw_ostream;
       Slot_Block,
 
       /// Early-clobber register use/def slot.  A live range defined at
-      /// Slot_EarlyClobber interferes with normal live ranges killed at
+      /// Slot_EarlyCLobber interferes with normal live ranges killed at
       /// Slot_Register.  Also used as the kill slot for live ranges tied to an
       /// early-clobber def.
       Slot_EarlyClobber,
@@ -108,6 +118,9 @@ class raw_ostream;
 
     PointerIntPair<IndexListEntry*, 2, unsigned> lie;
 
+    SlotIndex(IndexListEntry *entry, unsigned slot)
+      : lie(entry, slot) {}
+
     IndexListEntry* listEntry() const {
       assert(isValid() && "Attempt to compare reserved index.");
 #ifdef EXPENSIVE_CHECKS
@@ -134,12 +147,7 @@ class raw_ostream;
     };
 
     /// Construct an invalid index.
-    SlotIndex() = default;
-
-    // Creates a SlotIndex from an IndexListEntry and a slot. Generally should
-    // not be used. This method is only public to facilitate writing certain
-    // unit tests.
-    SlotIndex(IndexListEntry *entry, unsigned slot) : lie(entry, slot) {}
+    SlotIndex() : lie(nullptr, 0) {}
 
     // Construct a new slot index from the given one, and set the slot.
     SlotIndex(const SlotIndex &li, Slot s) : lie(li.listEntry(), unsigned(s)) {
@@ -147,7 +155,7 @@ class raw_ostream;
              "Attempt to construct index with 0 pointer.");
     }
 
-    /// Returns true if this is a valid index. Invalid indices do
+    /// Returns true if this is a valid index. Invalid indicies do
     /// not point into an index table, and cannot be compared.
     bool isValid() const {
       return lie.getPointer();
@@ -205,24 +213,14 @@ class raw_ostream;
       return A.listEntry()->getIndex() < B.listEntry()->getIndex();
     }
 
-    /// Return true if A refers to the same instruction as B or an earlier one.
-    /// This is equivalent to !isEarlierInstr(B, A).
-    static bool isEarlierEqualInstr(SlotIndex A, SlotIndex B) {
-      return !isEarlierInstr(B, A);
-    }
-
     /// Return the distance from this index to the given one.
     int distance(SlotIndex other) const {
       return other.getIndex() - getIndex();
     }
 
     /// Return the scaled distance from this index to the given one, where all
-    /// slots on the same instruction have zero distance, assuming that the slot
-    /// indices are packed as densely as possible. There are normally gaps
-    /// between instructions, so this assumption often doesn't hold. This
-    /// results in this function often returning a value greater than the actual
-    /// instruction distance.
-    int getApproxInstrDistance(SlotIndex other) const {
+    /// slots on the same instruction have zero distance.
+    int getInstrDistance(SlotIndex other) const {
       return (other.listEntry()->getIndex() - listEntry()->getIndex())
         / Slot_Count;
     }
@@ -274,7 +272,7 @@ class raw_ostream;
     SlotIndex getNextSlot() const {
       Slot s = getSlot();
       if (s == Slot_Dead) {
-        return SlotIndex(&*++listEntry()->getIterator(), Slot_Block);
+        return SlotIndex(listEntry()->getNextNode(), Slot_Block);
       }
       return SlotIndex(listEntry(), s + 1);
     }
@@ -282,7 +280,7 @@ class raw_ostream;
     /// Returns the next index. This is the index corresponding to the this
     /// index's slot, but for the next instruction.
     SlotIndex getNextIndex() const {
-      return SlotIndex(&*++listEntry()->getIterator(), getSlot());
+      return SlotIndex(listEntry()->getNextNode(), getSlot());
     }
 
     /// Returns the previous slot in the index list. This could be either the
@@ -294,7 +292,7 @@ class raw_ostream;
     SlotIndex getPrevSlot() const {
       Slot s = getSlot();
       if (s == Slot_Block) {
-        return SlotIndex(&*--listEntry()->getIterator(), Slot_Dead);
+        return SlotIndex(listEntry()->getPrevNode(), Slot_Dead);
       }
       return SlotIndex(listEntry(), s - 1);
     }
@@ -302,31 +300,50 @@ class raw_ostream;
     /// Returns the previous index. This is the index corresponding to this
     /// index's slot, but for the previous instruction.
     SlotIndex getPrevIndex() const {
-      return SlotIndex(&*--listEntry()->getIterator(), getSlot());
+      return SlotIndex(listEntry()->getPrevNode(), getSlot());
     }
+
   };
 
+  template <> struct isPodLike<SlotIndex> { static const bool value = true; };
+
   inline raw_ostream& operator<<(raw_ostream &os, SlotIndex li) {
     li.print(os);
     return os;
   }
 
-  using IdxMBBPair = std::pair<SlotIndex, MachineBasicBlock *>;
+  typedef std::pair<SlotIndex, MachineBasicBlock*> IdxMBBPair;
+
+  inline bool operator<(SlotIndex V, const IdxMBBPair &IM) {
+    return V < IM.first;
+  }
+
+  inline bool operator<(const IdxMBBPair &IM, SlotIndex V) {
+    return IM.first < V;
+  }
+
+  struct Idx2MBBCompare {
+    bool operator()(const IdxMBBPair &LHS, const IdxMBBPair &RHS) const {
+      return LHS.first < RHS.first;
+    }
+  };
 
   /// SlotIndexes pass.
   ///
   /// This pass assigns indexes to each instruction.
   class SlotIndexes : public MachineFunctionPass {
   private:
-    // IndexListEntry allocator.
-    BumpPtrAllocator ileAllocator;
 
-    using IndexList = ilist<IndexListEntry>;
+    typedef ilist<IndexListEntry> IndexList;
     IndexList indexList;
 
-    MachineFunction *mf = nullptr;
+#ifdef EXPENSIVE_CHECKS
+    IndexList graveyardList;
+#endif // EXPENSIVE_CHECKS
 
-    using Mi2IndexMap = DenseMap<const MachineInstr *, SlotIndex>;
+    MachineFunction *mf;
+
+    typedef DenseMap<const MachineInstr*, SlotIndex> Mi2IndexMap;
     Mi2IndexMap mi2iMap;
 
     /// MBBRanges - Map MBB number to (start, stop) indexes.
@@ -336,10 +353,14 @@ class raw_ostream;
     /// and MBB id.
     SmallVector<IdxMBBPair, 8> idx2MBBMap;
 
+    // IndexListEntry allocator.
+    BumpPtrAllocator ileAllocator;
+
     IndexListEntry* createEntry(MachineInstr *mi, unsigned index) {
       IndexListEntry *entry =
-          static_cast<IndexListEntry *>(ileAllocator.Allocate(
-              sizeof(IndexListEntry), alignof(IndexListEntry)));
+        static_cast<IndexListEntry*>(
+          ileAllocator.Allocate(sizeof(IndexListEntry),
+          alignOf<IndexListEntry>()));
 
       new (entry) IndexListEntry(mi, index);
 
@@ -352,9 +373,9 @@ class raw_ostream;
   public:
     static char ID;
 
-    SlotIndexes();
-
-    ~SlotIndexes() override;
+    SlotIndexes() : MachineFunctionPass(ID) {
+      initializeSlotIndexesPass(*PassRegistry::getPassRegistry());
+    }
 
     void getAnalysisUsage(AnalysisUsage &au) const override;
     void releaseMemory() override;
@@ -364,6 +385,9 @@ class raw_ostream;
     /// Dump the indexes.
     void dump() const;
 
+    /// Renumber the index list, providing space for new instructions.
+    void renumberIndexes();
+
     /// Repair indexes after adding and removing instructions.
     void repairIndexesInRange(MachineBasicBlock *MBB,
                               MachineBasicBlock::iterator Begin,
@@ -382,23 +406,14 @@ class raw_ostream;
 
     /// Returns true if the given machine instr is mapped to an index,
     /// otherwise returns false.
-    bool hasIndex(const MachineInstr &instr) const {
-      return mi2iMap.count(&instr);
+    bool hasIndex(const MachineInstr *instr) const {
+      return mi2iMap.count(instr);
     }
 
     /// Returns the base index for the given instruction.
-    SlotIndex getInstructionIndex(const MachineInstr &MI,
-                                  bool IgnoreBundle = false) const {
+    SlotIndex getInstructionIndex(const MachineInstr *MI) const {
       // Instructions inside a bundle have the same number as the bundle itself.
-      auto BundleStart = getBundleStart(MI.getIterator());
-      auto BundleEnd = getBundleEnd(MI.getIterator());
-      // Use the first non-debug instruction in the bundle to get SlotIndex.
-      const MachineInstr &BundleNonDebug =
-          IgnoreBundle ? MI
-                       : *skipDebugInstructionsForward(BundleStart, BundleEnd);
-      assert(!BundleNonDebug.isDebugInstr() &&
-             "Could not use a debug instruction to query mi2iMap.");
-      Mi2IndexMap::const_iterator itr = mi2iMap.find(&BundleNonDebug);
+      Mi2IndexMap::const_iterator itr = mi2iMap.find(getBundleStart(MI));
       assert(itr != mi2iMap.end() && "Instruction not found in maps.");
       return itr->second;
     }
@@ -412,11 +427,11 @@ class raw_ostream;
     /// Returns the next non-null index, if one exists.
     /// Otherwise returns getLastIndex().
     SlotIndex getNextNonNullIndex(SlotIndex Index) {
-      IndexList::iterator I = Index.listEntry()->getIterator();
+      IndexList::iterator I = Index.listEntry();
       IndexList::iterator E = indexList.end();
       while (++I != E)
         if (I->getInstr())
-          return SlotIndex(&*I, Index.getSlot());
+          return SlotIndex(I, Index.getSlot());
       // We reached the end of the function.
       return getLastIndex();
     }
@@ -424,15 +439,15 @@ class raw_ostream;
     /// getIndexBefore - Returns the index of the last indexed instruction
     /// before MI, or the start index of its basic block.
     /// MI is not required to have an index.
-    SlotIndex getIndexBefore(const MachineInstr &MI) const {
-      const MachineBasicBlock *MBB = MI.getParent();
-      assert(MBB && "MI must be inserted in a basic block");
+    SlotIndex getIndexBefore(const MachineInstr *MI) const {
+      const MachineBasicBlock *MBB = MI->getParent();
+      assert(MBB && "MI must be inserted inna basic block");
       MachineBasicBlock::const_iterator I = MI, B = MBB->begin();
-      while (true) {
+      for (;;) {
         if (I == B)
           return getMBBStartIdx(MBB);
         --I;
-        Mi2IndexMap::const_iterator MapItr = mi2iMap.find(&*I);
+        Mi2IndexMap::const_iterator MapItr = mi2iMap.find(I);
         if (MapItr != mi2iMap.end())
           return MapItr->second;
       }
@@ -441,15 +456,15 @@ class raw_ostream;
     /// getIndexAfter - Returns the index of the first indexed instruction
     /// after MI, or the end index of its basic block.
     /// MI is not required to have an index.
-    SlotIndex getIndexAfter(const MachineInstr &MI) const {
-      const MachineBasicBlock *MBB = MI.getParent();
-      assert(MBB && "MI must be inserted in a basic block");
+    SlotIndex getIndexAfter(const MachineInstr *MI) const {
+      const MachineBasicBlock *MBB = MI->getParent();
+      assert(MBB && "MI must be inserted inna basic block");
       MachineBasicBlock::const_iterator I = MI, E = MBB->end();
-      while (true) {
+      for (;;) {
         ++I;
         if (I == E)
           return getMBBEndIdx(MBB);
-        Mi2IndexMap::const_iterator MapItr = mi2iMap.find(&*I);
+        Mi2IndexMap::const_iterator MapItr = mi2iMap.find(I);
         if (MapItr != mi2iMap.end())
           return MapItr->second;
       }
@@ -487,75 +502,89 @@ class raw_ostream;
       return getMBBRange(mbb).second;
     }
 
-    /// Iterator over the idx2MBBMap (sorted pairs of slot index of basic block
-    /// begin and basic block)
-    using MBBIndexIterator = SmallVectorImpl<IdxMBBPair>::const_iterator;
-
-    /// Move iterator to the next IdxMBBPair where the SlotIndex is greater or
-    /// equal to \p To.
-    MBBIndexIterator advanceMBBIndex(MBBIndexIterator I, SlotIndex To) const {
-      return std::partition_point(
-          I, idx2MBBMap.end(),
-          [=](const IdxMBBPair &IM) { return IM.first < To; });
-    }
-
-    /// Get an iterator pointing to the IdxMBBPair with the biggest SlotIndex
-    /// that is greater or equal to \p Idx.
-    MBBIndexIterator findMBBIndex(SlotIndex Idx) const {
-      return advanceMBBIndex(idx2MBBMap.begin(), Idx);
-    }
-
-    /// Returns an iterator for the begin of the idx2MBBMap.
-    MBBIndexIterator MBBIndexBegin() const {
-      return idx2MBBMap.begin();
-    }
-
-    /// Return an iterator for the end of the idx2MBBMap.
-    MBBIndexIterator MBBIndexEnd() const {
-      return idx2MBBMap.end();
-    }
-
     /// Returns the basic block which the given index falls in.
     MachineBasicBlock* getMBBFromIndex(SlotIndex index) const {
       if (MachineInstr *MI = getInstructionFromIndex(index))
         return MI->getParent();
-
-      MBBIndexIterator I = findMBBIndex(index);
+      SmallVectorImpl<IdxMBBPair>::const_iterator I =
+        std::lower_bound(idx2MBBMap.begin(), idx2MBBMap.end(), index);
       // Take the pair containing the index
-      MBBIndexIterator J =
-        ((I != MBBIndexEnd() && I->first > index) ||
-         (I == MBBIndexEnd() && !idx2MBBMap.empty())) ? std::prev(I) : I;
+      SmallVectorImpl<IdxMBBPair>::const_iterator J =
+        ((I != idx2MBBMap.end() && I->first > index) ||
+         (I == idx2MBBMap.end() && idx2MBBMap.size()>0)) ? (I-1): I;
 
-      assert(J != MBBIndexEnd() && J->first <= index &&
+      assert(J != idx2MBBMap.end() && J->first <= index &&
              index < getMBBEndIdx(J->second) &&
              "index does not correspond to an MBB");
       return J->second;
     }
 
+    bool findLiveInMBBs(SlotIndex start, SlotIndex end,
+                        SmallVectorImpl<MachineBasicBlock*> &mbbs) const {
+      SmallVectorImpl<IdxMBBPair>::const_iterator itr =
+        std::lower_bound(idx2MBBMap.begin(), idx2MBBMap.end(), start);
+      bool resVal = false;
+
+      while (itr != idx2MBBMap.end()) {
+        if (itr->first >= end)
+          break;
+        mbbs.push_back(itr->second);
+        resVal = true;
+        ++itr;
+      }
+      return resVal;
+    }
+
+    /// Returns the MBB covering the given range, or null if the range covers
+    /// more than one basic block.
+    MachineBasicBlock* getMBBCoveringRange(SlotIndex start, SlotIndex end) const {
+
+      assert(start < end && "Backwards ranges not allowed.");
+
+      SmallVectorImpl<IdxMBBPair>::const_iterator itr =
+        std::lower_bound(idx2MBBMap.begin(), idx2MBBMap.end(), start);
+
+      if (itr == idx2MBBMap.end()) {
+        itr = std::prev(itr);
+        return itr->second;
+      }
+
+      // Check that we don't cross the boundary into this block.
+      if (itr->first < end)
+        return nullptr;
+
+      itr = std::prev(itr);
+
+      if (itr->first <= start)
+        return itr->second;
+
+      return nullptr;
+    }
+
     /// Insert the given machine instruction into the mapping. Returns the
     /// assigned index.
     /// If Late is set and there are null indexes between mi's neighboring
     /// instructions, create the new index after the null indexes instead of
     /// before them.
-    SlotIndex insertMachineInstrInMaps(MachineInstr &MI, bool Late = false) {
-      assert(!MI.isInsideBundle() &&
+    SlotIndex insertMachineInstrInMaps(MachineInstr *mi, bool Late = false) {
+      assert(!mi->isInsideBundle() &&
              "Instructions inside bundles should use bundle start's slot.");
-      assert(mi2iMap.find(&MI) == mi2iMap.end() && "Instr already indexed.");
-      // Numbering debug instructions could cause code generation to be
+      assert(mi2iMap.find(mi) == mi2iMap.end() && "Instr already indexed.");
+      // Numbering DBG_VALUE instructions could cause code generation to be
       // affected by debug information.
-      assert(!MI.isDebugInstr() && "Cannot number debug instructions.");
+      assert(!mi->isDebugValue() && "Cannot number DBG_VALUE instructions.");
 
-      assert(MI.getParent() != nullptr && "Instr must be added to function.");
+      assert(mi->getParent() != nullptr && "Instr must be added to function.");
 
-      // Get the entries where MI should be inserted.
+      // Get the entries where mi should be inserted.
       IndexList::iterator prevItr, nextItr;
       if (Late) {
-        // Insert MI's index immediately before the following instruction.
-        nextItr = getIndexAfter(MI).listEntry()->getIterator();
+        // Insert mi's index immediately before the following instruction.
+        nextItr = getIndexAfter(mi).listEntry();
         prevItr = std::prev(nextItr);
       } else {
-        // Insert MI's index immediately after the preceding instruction.
-        prevItr = getIndexBefore(MI).listEntry()->getIterator();
+        // Insert mi's index immediately after the preceding instruction.
+        prevItr = getIndexBefore(mi).listEntry();
         nextItr = std::next(prevItr);
       }
 
@@ -564,72 +593,73 @@ class raw_ostream;
       unsigned dist = ((nextItr->getIndex() - prevItr->getIndex())/2) & ~3u;
       unsigned newNumber = prevItr->getIndex() + dist;
 
-      // Insert a new list entry for MI.
+      // Insert a new list entry for mi.
       IndexList::iterator newItr =
-          indexList.insert(nextItr, createEntry(&MI, newNumber));
+        indexList.insert(nextItr, createEntry(mi, newNumber));
 
       // Renumber locally if we need to.
       if (dist == 0)
         renumberIndexes(newItr);
 
       SlotIndex newIndex(&*newItr, SlotIndex::Slot_Block);
-      mi2iMap.insert(std::make_pair(&MI, newIndex));
+      mi2iMap.insert(std::make_pair(mi, newIndex));
       return newIndex;
     }
 
-    /// Removes machine instruction (bundle) \p MI from the mapping.
-    /// This should be called before MachineInstr::eraseFromParent() is used to
-    /// remove a whole bundle or an unbundled instruction.
-    /// If \p AllowBundled is set then this can be used on a bundled
-    /// instruction; however, this exists to support handleMoveIntoBundle,
-    /// and in general removeSingleMachineInstrFromMaps should be used instead.
-    void removeMachineInstrFromMaps(MachineInstr &MI,
-                                    bool AllowBundled = false);
-
-    /// Removes a single machine instruction \p MI from the mapping.
-    /// This should be called before MachineInstr::eraseFromBundle() is used to
-    /// remove a single instruction (out of a bundle).
-    void removeSingleMachineInstrFromMaps(MachineInstr &MI);
+    /// Remove the given machine instruction from the mapping.
+    void removeMachineInstrFromMaps(MachineInstr *mi) {
+      // remove index -> MachineInstr and
+      // MachineInstr -> index mappings
+      Mi2IndexMap::iterator mi2iItr = mi2iMap.find(mi);
+      if (mi2iItr != mi2iMap.end()) {
+        IndexListEntry *miEntry(mi2iItr->second.listEntry());
+        assert(miEntry->getInstr() == mi && "Instruction indexes broken.");
+        // FIXME: Eventually we want to actually delete these indexes.
+        miEntry->setInstr(nullptr);
+        mi2iMap.erase(mi2iItr);
+      }
+    }
 
     /// ReplaceMachineInstrInMaps - Replacing a machine instr with a new one in
-    /// maps used by register allocator. \returns the index where the new
-    /// instruction was inserted.
-    SlotIndex replaceMachineInstrInMaps(MachineInstr &MI, MachineInstr &NewMI) {
-      Mi2IndexMap::iterator mi2iItr = mi2iMap.find(&MI);
+    /// maps used by register allocator.
+    void replaceMachineInstrInMaps(MachineInstr *mi, MachineInstr *newMI) {
+      Mi2IndexMap::iterator mi2iItr = mi2iMap.find(mi);
       if (mi2iItr == mi2iMap.end())
-        return SlotIndex();
+        return;
       SlotIndex replaceBaseIndex = mi2iItr->second;
       IndexListEntry *miEntry(replaceBaseIndex.listEntry());
-      assert(miEntry->getInstr() == &MI &&
+      assert(miEntry->getInstr() == mi &&
              "Mismatched instruction in index tables.");
-      miEntry->setInstr(&NewMI);
+      miEntry->setInstr(newMI);
       mi2iMap.erase(mi2iItr);
-      mi2iMap.insert(std::make_pair(&NewMI, replaceBaseIndex));
-      return replaceBaseIndex;
+      mi2iMap.insert(std::make_pair(newMI, replaceBaseIndex));
     }
 
     /// Add the given MachineBasicBlock into the maps.
-    /// If it contains any instructions then they must already be in the maps.
-    /// This is used after a block has been split by moving some suffix of its
-    /// instructions into a newly created block.
     void insertMBBInMaps(MachineBasicBlock *mbb) {
-      assert(mbb != &mbb->getParent()->front() &&
-             "Can't insert a new block at the beginning of a function.");
-      auto prevMBB = std::prev(MachineFunction::iterator(mbb));
+      MachineFunction::iterator nextMBB =
+        std::next(MachineFunction::iterator(mbb));
 
-      // Create a new entry to be used for the start of mbb and the end of
-      // prevMBB.
-      IndexListEntry *startEntry = createEntry(nullptr, 0);
-      IndexListEntry *endEntry = getMBBEndIdx(&*prevMBB).listEntry();
-      IndexListEntry *insEntry =
-          mbb->empty() ? endEntry
-                       : getInstructionIndex(mbb->front()).listEntry();
-      IndexList::iterator newItr =
-          indexList.insert(insEntry->getIterator(), startEntry);
+      IndexListEntry *startEntry = nullptr;
+      IndexListEntry *endEntry = nullptr;
+      IndexList::iterator newItr;
+      if (nextMBB == mbb->getParent()->end()) {
+        startEntry = &indexList.back();
+        endEntry = createEntry(nullptr, 0);
+        newItr = indexList.insertAfter(startEntry, endEntry);
+      } else {
+        startEntry = createEntry(nullptr, 0);
+        endEntry = getMBBStartIdx(nextMBB).listEntry();
+        newItr = indexList.insert(endEntry, startEntry);
+      }
 
       SlotIndex startIdx(startEntry, SlotIndex::Slot_Block);
       SlotIndex endIdx(endEntry, SlotIndex::Slot_Block);
 
+      MachineFunction::iterator prevMBB(mbb);
+      assert(prevMBB != mbb->getParent()->end() &&
+             "Can't insert a new block at the beginning of a function.");
+      --prevMBB;
       MBBRanges[prevMBB->getNumber()].second = startIdx;
 
       assert(unsigned(mbb->getNumber()) == MBBRanges.size() &&
@@ -638,15 +668,43 @@ class raw_ostream;
       idx2MBBMap.push_back(IdxMBBPair(startIdx, mbb));
 
       renumberIndexes(newItr);
-      llvm::sort(idx2MBBMap, less_first());
+      std::sort(idx2MBBMap.begin(), idx2MBBMap.end(), Idx2MBBCompare());
     }
+
+    /// \brief Free the resources that were required to maintain a SlotIndex.
+    ///
+    /// Once an index is no longer needed (for instance because the instruction
+    /// at that index has been moved), the resources required to maintain the
+    /// index can be relinquished to reduce memory use and improve renumbering
+    /// performance. Any remaining SlotIndex objects that point to the same
+    /// index are left 'dangling' (much the same as a dangling pointer to a
+    /// freed object) and should not be accessed, except to destruct them.
+    ///
+    /// Like dangling pointers, access to dangling SlotIndexes can cause
+    /// painful-to-track-down bugs, especially if the memory for the index
+    /// previously pointed to has been re-used. To detect dangling SlotIndex
+    /// bugs, build with EXPENSIVE_CHECKS=1. This will cause "erased" indexes to
+    /// be retained in a graveyard instead of being freed. Operations on indexes
+    /// in the graveyard will trigger an assertion.
+    void eraseIndex(SlotIndex index) {
+      IndexListEntry *entry = index.listEntry();
+#ifdef EXPENSIVE_CHECKS
+      indexList.remove(entry);
+      graveyardList.push_back(entry);
+      entry->setPoison();
+#else
+      indexList.erase(entry);
+#endif
+    }
+
   };
 
+
   // Specialize IntervalMapInfo for half-open slot index intervals.
   template <>
   struct IntervalMapInfo<SlotIndex> : IntervalMapHalfOpenInfo<SlotIndex> {
   };
 
-} // end namespace llvm
+}
 
 #endif // LLVM_CODEGEN_SLOTINDEXES_H
