@@ -16,28 +16,20 @@
 
 #include "llvm/ADT/SmallVector.h"
 #include "llvm/ADT/StringRef.h"
-#include <cassert>
-#include <cstddef>
-#include <cstdint>
-#include <cstring>
-#include <string>
+#include "llvm/Support/DataTypes.h"
 #include <system_error>
 
 namespace llvm {
-
-class formatv_object_base;
 class format_object_base;
 class FormattedString;
 class FormattedNumber;
-class FormattedBytes;
+template <typename T> class SmallVectorImpl;
 
 namespace sys {
 namespace fs {
-enum FileAccess : unsigned;
 enum OpenFlags : unsigned;
-enum CreationDisposition : unsigned;
-} // end namespace fs
-} // end namespace sys
+}
+}
 
 /// This class implements an extremely fast bulk output stream that can *only*
 /// output to a stream.  It does not support seeking, reopening, rewinding, line
@@ -45,6 +37,9 @@ enum CreationDisposition : unsigned;
 /// a chunk at a time.
 class raw_ostream {
 private:
+  void operator=(const raw_ostream &) = delete;
+  raw_ostream(const raw_ostream &) = delete;
+
   /// The buffer is handled in such a way that the buffer is
   /// uninitialized, unbuffered, or out of space when OutBufCur >=
   /// OutBufEnd. Thus a single comparison suffices to determine if we
@@ -74,7 +69,7 @@ private:
 public:
   // color order matches ANSI escape sequence, don't change
   enum Colors {
-    BLACK = 0,
+    BLACK=0,
     RED,
     GREEN,
     YELLOW,
@@ -91,9 +86,6 @@ public:
     OutBufStart = OutBufEnd = OutBufCur = nullptr;
   }
 
-  raw_ostream(const raw_ostream &) = delete;
-  void operator=(const raw_ostream &) = delete;
-
   virtual ~raw_ostream();
 
   /// tell - Return the current offset with the file.
@@ -192,7 +184,7 @@ public:
     return write(Str.data(), Str.length());
   }
 
-  raw_ostream &operator<<(const SmallVectorImpl<char> &Str) {
+  raw_ostream &operator<<(const llvm::SmallVectorImpl<char> &Str) {
     return write(Str.data(), Str.size());
   }
 
@@ -201,7 +193,6 @@ public:
   raw_ostream &operator<<(unsigned long long N);
   raw_ostream &operator<<(long long N);
   raw_ostream &operator<<(const void *P);
-
   raw_ostream &operator<<(unsigned int N) {
     return this->operator<<(static_cast<unsigned long>(N));
   }
@@ -215,12 +206,8 @@ public:
   /// Output \p N in hexadecimal, without any prefix or padding.
   raw_ostream &write_hex(unsigned long long N);
 
-  /// Output a formatted UUID with dash separators.
-  using uuid_t = uint8_t[16];
-  raw_ostream &write_uuid(const uuid_t UUID);
-
   /// Output \p Str, turning '\\', '\t', '\n', '"', and anything that doesn't
-  /// satisfy llvm::isPrint into an escape sequence.
+  /// satisfy std::isprint into an escape sequence.
   raw_ostream &write_escaped(StringRef Str, bool UseHexEscapes = false);
 
   raw_ostream &write(unsigned char C);
@@ -231,21 +218,13 @@ public:
 
   // Formatted output, see the leftJustify() function in Support/Format.h.
   raw_ostream &operator<<(const FormattedString &);
-
+  
   // Formatted output, see the formatHex() function in Support/Format.h.
   raw_ostream &operator<<(const FormattedNumber &);
-
-  // Formatted output, see the formatv() function in Support/FormatVariadic.h.
-  raw_ostream &operator<<(const formatv_object_base &);
-
-  // Formatted output, see the format_bytes() function in Support/Format.h.
-  raw_ostream &operator<<(const FormattedBytes &);
-
+  
   /// indent - Insert 'NumSpaces' spaces.
   raw_ostream &indent(unsigned NumSpaces);
 
-  /// write_zeros - Insert 'NumZeros' nulls.
-  raw_ostream &write_zeros(unsigned NumZeros);
 
   /// Changes the foreground color of text that will be output from this point
   /// forward.
@@ -267,7 +246,7 @@ public:
   /// outputting colored text, or before program exit.
   virtual raw_ostream &resetColor() { return *this; }
 
-  /// Reverses the foreground and background colors.
+  /// Reverses the forground and background colors.
   virtual raw_ostream &reverseColor() { return *this; }
 
   /// This function determines if this stream is connected to a "tty" or
@@ -298,6 +277,9 @@ private:
   /// \invariant { Size > 0 }
   virtual void write_impl(const char *Ptr, size_t Size) = 0;
 
+  // An out of line virtual method to provide a home for the class vtable.
+  virtual void handle();
+
   /// Return the current position within the stream, not counting the bytes
   /// currently in the buffer.
   virtual uint64_t current_pos() const = 0;
@@ -331,16 +313,13 @@ private:
   /// Copy data into the buffer. Size must not be greater than the number of
   /// unused bytes in the buffer.
   void copy_to_buffer(const char *Ptr, size_t Size);
-
-  virtual void anchor();
 };
 
 /// An abstract base class for streams implementations that also support a
-/// pwrite operation. This is useful for code that can mostly stream out data,
+/// pwrite operation. This is usefull for code that can mostly stream out data,
 /// but needs to patch in a header that needs to know the output size.
 class raw_pwrite_stream : public raw_ostream {
   virtual void pwrite_impl(const char *Ptr, size_t Size, uint64_t Offset) = 0;
-  void anchor() override;
 
 public:
   explicit raw_pwrite_stream(bool Unbuffered = false)
@@ -367,7 +346,13 @@ class raw_fd_ostream : public raw_pwrite_stream {
   int FD;
   bool ShouldClose;
 
-  std::error_code EC;
+  /// Error This flag is true if an error of any kind has been detected.
+  ///
+  bool Error;
+
+  /// Controls whether the stream should attempt to use atomic writes, when
+  /// possible.
+  bool UseAtomicWrites;
 
   uint64_t pos;
 
@@ -386,9 +371,7 @@ class raw_fd_ostream : public raw_pwrite_stream {
   size_t preferred_buffer_size() const override;
 
   /// Set the flag indicating that an output error has been encountered.
-  void error_detected(std::error_code EC) { this->EC = EC; }
-
-  void anchor() override;
+  void error_detected() { Error = true; }
 
 public:
   /// Open the specified file for writing. If an error occurs, information
@@ -397,22 +380,15 @@ public:
   /// \p Flags allows optional flags to control how the file will be opened.
   ///
   /// As a special case, if Filename is "-", then the stream will use
-  /// STDOUT_FILENO instead of opening a file. This will not close the stdout
-  /// descriptor.
-  raw_fd_ostream(StringRef Filename, std::error_code &EC);
+  /// STDOUT_FILENO instead of opening a file. Note that it will still consider
+  /// itself to own the file descriptor. In particular, it will close the
+  /// file descriptor when it is done (this is necessary to detect
+  /// output errors).
   raw_fd_ostream(StringRef Filename, std::error_code &EC,
-                 sys::fs::CreationDisposition Disp);
-  raw_fd_ostream(StringRef Filename, std::error_code &EC,
-                 sys::fs::FileAccess Access);
-  raw_fd_ostream(StringRef Filename, std::error_code &EC,
-                 sys::fs::OpenFlags Flags);
-  raw_fd_ostream(StringRef Filename, std::error_code &EC,
-                 sys::fs::CreationDisposition Disp, sys::fs::FileAccess Access,
                  sys::fs::OpenFlags Flags);
 
   /// FD is the file descriptor that this writes to.  If ShouldClose is true,
-  /// this closes the file when the stream is destroyed. If FD is for stdout or
-  /// stderr, it will not be closed.
+  /// this closes the file when the stream is destroyed.
   raw_fd_ostream(int fd, bool shouldClose, bool unbuffered=false);
 
   ~raw_fd_ostream() override;
@@ -427,6 +403,16 @@ public:
   /// to the offset specified from the beginning of the file.
   uint64_t seek(uint64_t off);
 
+  /// Set the stream to attempt to use atomic writes for individual output
+  /// routines where possible.
+  ///
+  /// Note that because raw_ostream's are typically buffered, this flag is only
+  /// sensible when used on unbuffered streams which will flush their output
+  /// immediately.
+  void SetUseAtomicWrites(bool Value) {
+    UseAtomicWrites = Value;
+  }
+
   raw_ostream &changeColor(enum Colors colors, bool bold=false,
                            bool bg=false) override;
   raw_ostream &resetColor() override;
@@ -437,13 +423,13 @@ public:
 
   bool has_colors() const override;
 
-  std::error_code error() const { return EC; }
-
   /// Return the value of the flag in this raw_fd_ostream indicating whether an
   /// output error has been encountered.
   /// This doesn't implicitly flush any pending output.  Also, it doesn't
   /// guarantee to detect all errors unless the stream has been closed.
-  bool has_error() const { return bool(EC); }
+  bool has_error() const {
+    return Error;
+  }
 
   /// Set the flag read by has_error() to false. If the error flag is set at the
   /// time when this raw_ostream's destructor is called, report_fatal_error is
@@ -454,7 +440,9 @@ public:
   ///    Unless explicitly silenced."
   ///      - from The Zen of Python, by Tim Peters
   ///
-  void clear_error() { EC = std::error_code(); }
+  void clear_error() {
+    Error = false;
+  }
 };
 
 /// This returns a reference to a raw_ostream for standard output. Use it like:
@@ -483,7 +471,6 @@ class raw_string_ostream : public raw_ostream {
   /// Return the current position within the stream, not counting the bytes
   /// currently in the buffer.
   uint64_t current_pos() const override { return OS.size(); }
-
 public:
   explicit raw_string_ostream(std::string &O) : OS(O) {}
   ~raw_string_ostream() override;
@@ -498,9 +485,6 @@ public:
 
 /// A raw_ostream that writes to an SmallVector or SmallString.  This is a
 /// simple adaptor class. This class does not encounter output errors.
-/// raw_svector_ostream operates without a buffer, delegating all memory
-/// management to the SmallString. Thus the SmallString is always up-to-date,
-/// may be used directly and there is no need to call flush().
 class raw_svector_ostream : public raw_pwrite_stream {
   SmallVectorImpl<char> &OS;
 
@@ -509,24 +493,32 @@ class raw_svector_ostream : public raw_pwrite_stream {
 
   void pwrite_impl(const char *Ptr, size_t Size, uint64_t Offset) override;
 
-  /// Return the current position within the stream.
+  /// Return the current position within the stream, not counting the bytes
+  /// currently in the buffer.
   uint64_t current_pos() const override;
 
+protected:
+  // Like the regular constructor, but doesn't call init.
+  explicit raw_svector_ostream(SmallVectorImpl<char> &O, unsigned);
+  void init();
+
 public:
   /// Construct a new raw_svector_ostream.
   ///
   /// \param O The vector to write to; this should generally have at least 128
   /// bytes free to avoid any extraneous memory overhead.
-  explicit raw_svector_ostream(SmallVectorImpl<char> &O) : OS(O) {
-    SetUnbuffered();
-  }
+  explicit raw_svector_ostream(SmallVectorImpl<char> &O);
+  ~raw_svector_ostream() override;
 
-  ~raw_svector_ostream() override = default;
 
-  void flush() = delete;
+  /// This is called when the SmallVector we're appending to is changed outside
+  /// of the raw_svector_ostream's control.  It is only safe to do this if the
+  /// raw_svector_ostream has previously been flushed.
+  void resync();
 
-  /// Return a StringRef for the vector contents.
-  StringRef str() { return StringRef(OS.data(), OS.size()); }
+  /// Flushes the stream contents to the target vector and return a StringRef
+  /// for the vector contents.
+  StringRef str();
 };
 
 /// A raw_ostream that discards all output.
@@ -540,7 +532,7 @@ class raw_null_ostream : public raw_pwrite_stream {
   uint64_t current_pos() const override;
 
 public:
-  explicit raw_null_ostream() = default;
+  explicit raw_null_ostream() {}
   ~raw_null_ostream() override;
 };
 
@@ -549,10 +541,12 @@ class buffer_ostream : public raw_svector_ostream {
   SmallVector<char, 0> Buffer;
 
 public:
-  buffer_ostream(raw_ostream &OS) : raw_svector_ostream(Buffer), OS(OS) {}
-  ~buffer_ostream() override { OS << str(); }
+  buffer_ostream(raw_ostream &OS) : raw_svector_ostream(Buffer, 0), OS(OS) {
+    init();
+  }
+  ~buffer_ostream() { OS << str(); }
 };
 
-} // end namespace llvm
+} // end llvm namespace
 
-#endif // LLVM_SUPPORT_RAW_OSTREAM_H
+#endif
